from mcd.base import Mcd
from typing import NamedTuple
from dataclasses import dataclass

@dataclass
class Coder:
    encoder: dict[str, int]
    decoder: list[str]


@dataclass
class Data_packs:
    download: Coder = None
    upload: Coder = None


class DefaultDataPacks(NamedTuple):
    handshaking: Data_packs
    status: Data_packs
    login: Data_packs
    play: Data_packs


class PacketsMcd(Mcd):
    datapacks: DefaultDataPacks

    def __init__(self, version: str) -> None:
        super(PacketsMcd, self).__init__(version)
        self.datapacks = self._create_datapacks()

    def _create_datapacks(self):
        datapacks = []

        for name in DefaultDataPacks._fields:
            state = self.data['protocol'][name]
            dp = Data_packs()
            
            for direct, direct2 in zip(('toClient', 'toServer'), ('download', 'upload')):
                enc = {}
                dec = []
                
                for id_packet, name_packet in state[direct]['types']['packet'][1][0]['type'][1]['mappings'].items():
                    id_packet = int(id_packet, 16)
                    enc[name_packet] = id_packet
                    dec.append(name_packet)
                setattr(dp, direct2, Coder(enc, dec))
            
            datapacks.append(dp)
        return DefaultDataPacks(*datapacks)








